/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import { splitTokens } from '@shikijs/core'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, Text, type TextStyle, View, type ViewStyle } from 'react-native'
import php from 'shiki/dist/langs/php.mjs'
import tsx from 'shiki/dist/langs/tsx.mjs'
import typescript from 'shiki/dist/langs/typescript.mjs'
import { getHighlighter } from './SyntaxContext'

const LANGS = {
  php,
  tsx,
  typescript,
  // Add more languages as needed
} as const

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[SyntaxHighlighter] ${message}`, ...args)
  }
}

function extractTokenPatterns(tokens: ThemedToken[][]) {
  const breakpoints = new Set<number>()

  tokens.forEach((line) => {
    line.forEach((token) => {
      // Add token boundaries
      const start = token.offset || 0
      const end = start + token.content.length
      breakpoints.add(start)
      breakpoints.add(end)

      // Get explanation for token type
      if (token.explanation) {
        token.explanation.forEach((exp) => {
          // Add breakpoints for each scope
          exp.scopes.forEach((scope) => {
            // Add scope-specific patterns
            const match = scope.scopeName?.match(/\b\w+\b/g)
            if (match) {
              match.forEach((word) => {
                const idx = token.content.indexOf(word)
                if (idx !== -1) {
                  breakpoints.add(start + idx)
                  breakpoints.add(start + idx + word.length)
                }
              })
            }
          })
        })
      }

      // Add breakpoints for operators and delimiters
      const operators = token.content.match(/([(){}[\]<>.,;:?=+\-*/%&|^!~`@#$]+)/g)
      if (operators) {
        operators.forEach((op) => {
          const idx = token.content.indexOf(op)
          if (idx !== -1) {
            breakpoints.add(start + idx)
            breakpoints.add(start + idx + op.length)
          }
        })
      }

      // Add breakpoints for keywords
      const keywords = token.content.match(/\b(function|const|let|var|return|if|else|for|while|class|extends|import|export|default|void|null|undefined|true|false)\b/g)
      if (keywords) {
        keywords.forEach((keyword) => {
          const idx = token.content.indexOf(keyword)
          if (idx !== -1) {
            breakpoints.add(start + idx)
            breakpoints.add(start + idx + keyword.length)
          }
        })
      }

      // Add breakpoints for strings
      const strings = token.content.match(/(['"`])(?:(?!\1)[^\\]|\\.)*\1/g)
      if (strings) {
        strings.forEach((str) => {
          const idx = token.content.indexOf(str)
          if (idx !== -1) {
            breakpoints.add(start + idx)
            breakpoints.add(start + idx + str.length)
          }
        })
      }
    })
  })

  return breakpoints
}

function processTokens(tokens: ThemedToken[][], language: string): ThemedToken[][] {
  // Get token patterns from the existing tokenization
  const tokenBreakpoints = extractTokenPatterns(tokens)

  // Get the language grammar for additional context
  const grammarArray = LANGS[language as keyof typeof LANGS]
  if (!grammarArray?.length) {
    log('No grammar found for language:', language)
    return tokens
  }

  // Use the first grammar in the array (most languages only have one)
  const grammar = grammarArray[0]

  // Add breakpoints from grammar patterns
  if (grammar?.repository) {
    Object.values(grammar.repository).forEach((repo: any) => {
      if (repo.patterns) {
        repo.patterns.forEach((pattern: any) => {
          // Handle pattern matches
          if (pattern.match) {
            try {
              const regex = new RegExp(pattern.match, 'g')
              tokens.forEach((line) => {
                line.forEach((token) => {
                  const matches = token.content.match(regex)
                  if (matches) {
                    matches.forEach((m) => {
                      const idx = token.content.indexOf(m)
                      if (idx !== -1) {
                        tokenBreakpoints.add((token.offset || 0) + idx)
                        tokenBreakpoints.add((token.offset || 0) + idx + m.length)
                      }
                    })
                  }
                })
              })
            }
            catch {
              // Ignore invalid regex patterns from grammar
              log('Invalid pattern regex:', pattern.match)
            }
          }

          // Handle begin/end patterns
          if (pattern.begin) {
            try {
              const regex = new RegExp(pattern.begin, 'g')
              tokens.forEach((line) => {
                line.forEach((token) => {
                  const matches = token.content.match(regex)
                  if (matches) {
                    matches.forEach((m) => {
                      const idx = token.content.indexOf(m)
                      if (idx !== -1) {
                        tokenBreakpoints.add((token.offset || 0) + idx)
                        tokenBreakpoints.add((token.offset || 0) + idx + m.length)
                      }
                    })
                  }
                })
              })
            }
            catch {
              log('Invalid begin regex:', pattern.begin)
            }
          }
        })
      }
    })
  }

  // Use Shiki's splitTokens with our collected breakpoints
  return splitTokens(tokens, tokenBreakpoints)
}

export interface SyntaxHighlighterProps {
  text: string
  language: string
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  highlightLines?: number[]
}

const defaultHighlightLines: number[] = []

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines,
}) => {
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])

  React.useEffect(() => {
    const highlight = async () => {
      try {
        log('Starting highlighting process')
        log('Input:', { text, language, theme: theme.name })

        setLoading(true)
        const highlighter = await getHighlighter()
        log('Highlighter instance obtained')

        if (!text || !language || !theme) {
          throw new Error('Missing required props')
        }

        log('Attempting tokenization...')
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme,
          includeExplanation: true,
        })

        log('Raw tokenization result:', result)

        if (!result?.tokens) {
          throw new Error('Invalid tokenization result')
        }

        // Process tokens using the loaded grammar
        const processedTokens = await processTokens(result.tokens, language)
        log('Processed tokens:', processedTokens)

        setTokens(processedTokens)
        setError(null)
      }
      catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code'
        log('Error during highlighting:', err)
        setError(errorMessage)
        setTokens([])
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    return (
      <Text
        key={`${token.offset || index}-${index}`}
        style={[
          {
            color: token.color,
            fontSize,
            fontFamily: 'monospace',
            fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
            fontWeight: token.fontStyle === 1 ? 'bold' : 'normal',
          },
          textStyle,
        ]}
      >
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => {
    const isHighlighted = highlightLines.includes(index + 1)
    return (
      <View
        key={index}
        style={[
          {
            flexDirection: 'row',
            flexWrap: 'wrap',
            backgroundColor: isHighlighted ? 'rgba(255,255,255,0.1)' : 'transparent',
          },
        ]}
      >
        {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
      </View>
    )
  }

  if (loading) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor: theme.colors?.background || '#1e1e1e' }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 10 }, containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}
